#!/bin/bash
#
# This script setup the database for collector
#
# Author: Abilis <support@antek.it>
# Date:   04/06/2015
# Ver:    3.1.1-1
#
# Changelog
# 3.1.1: Fixed field in port_type
# 3.1.0: Added new fields for port_type
# 3.0.0: Fixed privileges grant
# 2.1.0: Fixed an error in codes table (ItoS was 36 instead of 37)
# 2.0.0: First release of the install script
#

# --- CONSTANTS DEFINITIONS ---

# Get full path of the script

SCRIPT_FULL_PATH=`readlink -f $0`
SCRIPT_DIR_PATH=`dirname ${SCRIPT_FULL_PATH}`

source ${SCRIPT_DIR_PATH}/include/const.sh

# SQL setup file
DB_SQL="${SCRIPT_DIR_PATH}/sql/setup.sql"

# --- END CONSTANTS DEFINITIONS ---

echo
echo "#################################"
echo "#  Collector database install   #"
echo "#           Ver. ${VER}        #"
echo "#              ***              #"
echo "#    Abilis: www.abilis.net     #"
echo "#################################"

# Check if the install script is running as root
if [[ ${UID} != "0" ]]; then
  echo -e "ERROR: You need to be root to install the collector database\n"
  exit 1
fi

# Check if CONFFILE exists
if [[ ! -e ${CONFFILE} ]]; then
  echo -e "\nERROR: File ${CONFFILE} not found\n"
  echo "If collector was installed into a different place please specify the"
  echo "collector.conf path into the CONFFILE variable inside this script."
  echo
  exit 1
fi

# Just make a precheck before begin installation
source ${SCRIPT_DIR_PATH}/precheck.sh 

# Obtain database info from the CONFFILE
MYSQLHOST=`grep MYSQLHOST ${CONFFILE} | awk -F ':' '{print $2}'`
MYSQLPORT=`grep MYSQLPORT ${CONFFILE} | awk -F ':' '{print $2}'`
MYSQLDB=`grep MYSQLDB ${CONFFILE} | awk -F ':' '{print $2}'`
MYSQLUSER=`grep MYSQLUSER ${CONFFILE} | awk -F ':' '{print $2}'`
MYSQLPASS=`grep MYSQLPASS ${CONFFILE} | awk -F ':' '{print $2}'`
MYSQLTABLES=`egrep ^MYSQLTABLE ${CONFFILE} | sort | uniq | awk -F ':' '{print $2}'`

# Check if required database fields are present
for keyword in MYSQLHOST MYSQLDB MYSQLUSER; do
  if [[ -z ${!keyword} ]]; then
    echo -e "\nERROR: ${keyword} is not specified in ${CONFFILE}!"
    echo -e "Please set the correct db credentials in ${CONFFILE}\n"
    exit 2
  fi
done

# Set default database port if not set into collector.conf
if [[ -z ${MYSQLPORT} ]]; then
  echo -e "INFO: MYSQLPORT is not set, assuming 3306\m"
  MYSQLPORT=3306
fi

# Assume no password if MYSQLPASS is not set
if [[ -z ${MYSQLPASS} ]]; then
  echo -e "INFO: MYSQLPASS is not set\n"
fi

dba_user=""
while [[ 1 ]]; do
  echo -en "Please enter the username of the database administrator (usually root).\n"
  echo -en "It will be used ONLY for installation: "
  read dba_user;

  if [[ -z ${dba_user} ]]; then 
    echo -e "ERROR: username cannot be empty\n"
  else 
    break;
  fi
done

dba_pass=""
echo -e "\nPlease enter the database password for ${dba_user}"
echo -en "or leave empty if none (password will not be echoed): "
stty -echo 
read dba_pass;
stty echo

if [[ ! -z ${dba_pass} ]]; then
  dba_pass="-p${dba_pass}"
fi

# Summary
echo
echo
echo "###############"
echo "#   Summary   #"
echo "###############"
echo
echo -e "MySQL Admin User:  ${dba_user}"
echo
echo -e "MySQL Host:        ${MYSQLHOST}"
echo -e "MySQL Port:        ${MYSQLPORT}"
echo -e "MySQL DB:          ${MYSQLDB}"
echo -e "MySQL User:        ${MYSQLUSER}"
echo

choice=""
while [[ ! ${choice} =~ (^yes$)|(^no$) ]]; do  
  echo -n "Do you want to continue (yes/no)? " 
  read choice 
  echo 

  if [[ ${choice} == "yes" ]]; then 
    echo -e "*** Beginning installation ***\n" 
  elif [[ ${choice} == "no" ]]; then 
    echo -e "*** Installation aborted! ***\n" 
    exit 1 
  else 
    echo -e "Please choose either yes or no" 
  fi 
done

MYSQL_CMD="mysql -u${dba_user} ${dba_pass}"

# Create the database
echo -n "Creating Database... "
${MYSQL_CMD} -e "CREATE DATABASE ${MYSQLDB}"

if [[ $? -eq 0 ]]; then
  echo "Done!"
else
  echo -e "\nERROR: Database creation fails! Aborting.\n"
  exit 255
fi

# Granting privileges to ${MYSQLUSER} on ${MYSQLDB} 
# ONLY IF ${MYSQLUSER} != root
if [[ ${MYSQLUSER} != "root" ]]; then
  echo -n "Granting privileges to ${MYSQLUSER} on database ${MYSQLDB}... "

  grant_query="GRANT ALTER,INSERT,SELECT,CREATE,DROP,LOCK TABLES,UPDATE ON ${MYSQLDB}.*"
  grant_query="${grant_query} TO '${MYSQLUSER}'@'${MYSQLHOST}'"

  if [[ ! -z ${MYSQLPASS} ]]; then
    grant_query="${grant_query} IDENTIFIED BY '${MYSQLPASS}'"
  fi

  ${MYSQL_CMD} -e "${grant_query}"

  if [[ $? -eq 0 ]]; then
    echo "Done!"
  else
    echo -e "\nERROR: Granting user privileges failed! Aborting.\n"
    exit 255
  fi
fi

# Importing database
echo -n "Importing database... "

${MYSQL_CMD} ${MYSQLDB} < ${DB_SQL}

if [[ $? -eq 0 ]]; then
  echo "Done!"
else
  echo -e "\nERROR: Database import failed.\n"
  exit 255
fi

# END INSTALLING

echo -e "\n### All done! ###"
echo
