#!/bin/bash
#
# Abilis Collector Installer
#
# Author: Dario Fiumicello <fiumicello@antek.it>
# Ver: 3.1.1-1
#
# Copyright (c) 2015 - Abilis


# Do not change those constants since they are coded also into collector code
# If you need to change them please recompile the collector
INSTALL_DIR=/opt/abilis
BIN_DIR=${INSTALL_DIR}/bin
ETC_DIR=/etc/opt/abilis
LOG_DIR=/var/opt/abilis/log/collector
RUN_DIR=/var/opt/abilis/run
SHARE_DIR=/opt/abilis/share/collector
VERSION=3.1.1

INST_PROGRAM_FILE=collector
INST_CONF_FILE=collector.conf
INST_INIT_FILE=collector_init
INST_UNINST_FILE=uninstall.sh
INST_DB_DIR=database
INST_VIEWER_DIR=viewer
README_FILE=README
VERSION_FILE=VERSION

START_LEVELS="2 3 4 5"
KILL_LEVELS="0 1 6"

BASH=${BASH:=`which bash`} || (echo "FATAL: bash not found!"; exit 1)
SED=`which sed` || (echo "FATAL: sed not found!"; exit 1)
DIRNAME=`which dirname` || (echo "FATAL: dirname not found!"; exit 1)
RM=`which rm` || (echo "FATAL: rm not found!"; exit 1)
PIDOF=`which pidof` || (echo "FATAL: pidof not found!"; exit 1)
STTY=`which stty` || (echo "FATAL: stty not found!"; exit 1)

### FUNCTIONS DEFINITIONS ###

#
# function ask_user
#
# Ask a question to the user and force him to choose between a set of possible
# answers.
#
# Params:
# 1) The question to prompt to the user
# 2) A string with possible answers separated by a space
# 3) A variable which will contain the answer
#
# Returns always 0
#
function ask_user {

  if [[ $# != 3 ]]; then
    echo "ask_user: FATAL! 3 argouments are needed"
    exit 255
  fi

  local question=${1}
  local answers=${2}
  local result=${3}

  regex=`echo ${answers} | ${SED} 's/ /|/g'`

  local user_answer
  while [[ true ]]; do
    echo -en "${question} "
    read user_answer
    if [[ "${user_answer}" =~ ${regex} ]]; then
      eval $result="'${user_answer}'";
      return 0
    else
      echo "Please choose among \"$answers\""
    fi
  done
}

#
# func check_return_code
#
# Check if a program exitcode is equal or different than 0. If it is equal
# prints "[OK]\n", otherwise prints "[FAIL]\n" followed by an error message
# specified by the user
#
# Params:
# 1) The exitcode to check
# 2) (optional) The message to print in case of error
#
# Returns the passed exitcode
#
function check_return_code {
  if [[ $# -gt 2 ]]; then
    echo "check_return_code: FATAL! no more than 2 argouments are accepted"
    exit 255
  fi

  local code=${1}
  local errmsg=${2}

  if [[ ${code} == "0" ]]; then
    echo "[OK]"
  else
    echo -e "\n$errmsg"
  fi

  return ${code}
}

### END FUNCTIONS DEFINITIONS ###



### MAIN PROGRAM ###

echo
echo "################################"
echo "#    Abilis Linux Collector    #"
echo "#          Installer           #"
echo "#             ---              #"
echo "#    (c) 2015 - abilis.net     #"
echo "################################"
echo

# Check if the install script is running as root
if [[ ${UID} != "0" ]]; then
  echo -e "ERROR: You need to be root to install the collector\n"
  exit 1
fi

# Check if a collector is still running
${PIDOF} collector > /dev/null
if [[ $? == "0" ]]; then
  echo "ERROR: A collector is already running!"
  echo -e "Please stop it before installing the new collector.\n"
  exit 1
fi

# Check if a collector is already present in ${BIN_DIR}
if [[ -x "${BIN_DIR}/collector" ]]; then
  echo -e "A collector installation is already present in ${BIN_DIR}/collector\n"
  ask_user \
    "Do you want to upgrade it? (y/n)" \
    "y n" \
    upgrade_collector

  if [[ "${upgrade_collector}" == "y" ]]; then
    ./upgrade.sh
    exit $?
  fi
  echo -e "\nIf you want to install a fresh new collector please uninstall"
  echo -e "the old one first, using ${SHARE_DIR}/${INST_UNINST_FILE}"
  echo -e "\nIf you want to upgrade it you can run ./upgrade.sh script\n"
  exit 1
fi

echo -n "Testing init scripts path... "
INIT_DIR=
for dir in /etc/init.d /etc/rc.d/init.d; do
	if [ -d $dir ]; then
		INIT_DIR=$dir
    echo -e "${INIT_DIR}\n"
		break
	fi
done

run_at_boot=
if [[ -z "${INIT_DIR}" ]]; then
  echo "FAILED!" 
  echo
  echo "Init dir not found, you should copy the collector_init in the"
  echo "appropriate place for your distro"
  echo
  run_at_boot="n"
else
  # Ask the user if he wants to run collector at boot
  ask_user \
    "Do you want to run collector at boot time? (y/n)" \
    "y n" \
    run_at_boot
fi

# Verify if a configuration already exists
use_prev_conf=""
etc_collector_cc_path=/etc/collector/collector.conf
etc_opt_cc_path=/etc/opt/abilis/collector.conf

if [[ -e "${etc_collector_cc_path}" && -e "${etc_opt_cc_path}" ]]; then
  echo -e "\nFound two old collector.conf files:"
  echo -e "\t1) ${etc_collector_cc_path}"
  echo -e "\t2) ${etc_opt_cc_path}\n"
  ask_user \
    "Do you want to use one of them? (y/n)" \
    "y n" \
    use_one_of_them

  if [[ "${use_one_of_them}" == "y" ]]; then
    ask_user \
      "Which one do you want to use? (1/2)" \
      "1 2" \
      which_one
      
      if [[ "${which_one}" == "1" ]]; then
        INST_CONF_FILE="${etc_collector_cc_path}"
      else
        INST_CONF_FILE="${etc_opt_cc_path}"
      fi
    
      use_prev_conf="y"
  fi

elif [[ -e "${etc_collector_cc_path}" ]]; then
  echo -e "\nFound an old configuration file: ${etc_collector_cc_path}."
  ask_user \
    "Do you want to use it? (y/n)" \
    "y n" \
    use_it

  if [[ "${use_it}" == "y" ]]; then
    INST_CONF_FILE="${etc_collector_cc_path}"
    use_prev_conf="y"
  fi

elif [[ -e "${etc_opt_cc_path}" ]]; then
  echo -e "\nFound a configuration file: ${etc_opt_cc_path}"
  ask_user \
    "Do you want to use it? (y/n)" \
    "y n" \
    use_it

  if [[ "${use_it}" == "y" ]]; then
    INST_CONF_FILE="${etc_opt_cc_path}"
    use_prev_conf="y"
  fi

fi

if [[ -e "${etc_collector_cc_path}" ]]; then
  # Warn the user about the presence of the old collector
  ${STTY} -echo
  echo -e "\n*** WARNING ***\n"
  echo -e "An old collector version was found in /etc/collector!\n"
  echo "Even if it will not conflict with the new collector it is strongly"
  echo "suggested to remove it (or save it elsewhere) in order to avoid"
  echo "confusion!"
  echo -e "\n***************\n"
  echo "If you have read the previous message press a key to continue"
  read dummy
  ${STTY} echo
fi

### Installation summary ###

echo -e "\n*** Installation summary ***"
echo
echo "DIRECTORIES"
echo "  Main directory:           ${INSTALL_DIR}"
echo "  Binary directory:         ${BIN_DIR}"
echo "  Config files directory:   ${ETC_DIR}"
echo "  Log directory:            ${LOG_DIR}"
echo "  Database stuff direcotry: ${SHARE_DIR}/${INST_DB_DIR}"
echo "  Viewer direcotry:         ${SHARE_DIR}/${INST_VIEWER_DIR}"
echo
echo -n "The collector will "
if [[ "${run_at_boot}" == "n" ]]; then
  echo -n "NOT "
fi
echo "be run at boot!"
echo
if [[ "${use_prev_conf}" == "y" ]]; then
  echo -e "Using previous config file: ${INST_CONF_FILE}"
  echo
fi
if [[ -z "${INIT_DIR}" ]]; then
  echo -e "Init directory wasn't found!"
  echo
fi
echo "***"
echo
ask_user \
  "Do you want to confirm and start installation? (yes/no)" \
  "yes no" \
  confirm_and_install

if [[ "${confirm_and_install}" == "no" ]]; then
  echo -e "\nAborting installation as requested! Bye bye.\n"
  exit 1
fi

###  Installation begins ###

echo -e "\n*** Installation begins ***\n"

# If we should use previous conf then save the file elsewhere
if [[ "${use_prev_conf}" == "y" ]]; then
  echo -n "Saving ${INST_CONF_FILE} "
  cp "${INST_CONF_FILE}" /tmp/collector.conf
  check_return_code \
    $? \
    "Error while saving previus conf into /tmp/collector.conf. I quit!" || \
    exit $?
  INST_CONF_FILE=/tmp/collector.conf
  echo
fi

# Creating installation directory
echo "Creating installation directories... "
for dir in \
  ${INSTALL_DIR} ${BIN_DIR} ${ETC_DIR} ${LOG_DIR} ${RUN_DIR} ${SHARE_DIR}; do
  echo -en "\t${dir} "
  mkdir -p "${dir}"
  check_return_code \
    $? \
    "Error while creating directory ${dir}. I have to quit! :(" || exit $?
done

echo "Directories created successfully!"

# Copying files
echo -en "\nCopying program files... "
cp -ax ${INST_PROGRAM_FILE} ${BIN_DIR}/ && \
cp -ax ${INST_CONF_FILE} ${ETC_DIR}/ && \
cp -ax ${INST_INIT_FILE} ${ETC_DIR}/ && \
cp -ax ${README_FILE} ${SHARE_DIR}/ && \
echo ${VERSION} > ${SHARE_DIR}/${VERSION_FILE} && \
cp -ax ${INST_UNINST_FILE} ${SHARE_DIR}/ && \
chmod 750 ${SHARE_DIR}/${INST_UNINST_FILE} && \
cp -ax ${INST_DB_DIR} ${SHARE_DIR}/ && \
cp -ax ${INST_VIEWER_DIR} ${SHARE_DIR}/ && \
chmod -R 755 ${SHARE_DIR}/${INST_VIEWER_DIR}
  check_return_code \
    $? \
    "Error while copying installation files. Sorry! :(" ||  exit 1

# Eventually copy collector_init in ${INIT_DIR}
if [[ -d "${INIT_DIR}" ]]; then
  echo -en "\nCopying collector init file int ${INIT_DIR}... "
  cp -ax ${INST_INIT_FILE} ${INIT_DIR}/collector && \
  chmod +x ${INIT_DIR}/collector
  check_return_code \
    $? \
    "Error while creating ${INIT_DIR}/collector. I have to quitt :(" ||  exit 1
fi

# If user choose to run collector at boot then install the rc scripts
if [[ "${run_at_boot}" == "y" && -x "${INIT_DIR}/collector" ]]; then
  # Check if we are on a Debian-based or Redhat-based system
  UPDATE_RCD=`which update-rc.d 2>/dev/null` # Debian-based systems use update-rc.d
  CHKCONFIG=`which chkconfig 2>/dev/null` # Redhat-based systems use chkconfig

  if [[ -x "${UPDATE_RCD}" ]]; then
    echo -e "\nSystem is Debian-based, using update-rc.d for boot files installation"
    ${UPDATE_RCD} collector start 99 ${START_LEVELS} . stop 15 ${KILL_LEVELS} .
    check_return_code \
      $? \
      "WARNING: update-rc.d failed, you should manually configure collector to run at boot!"
    
  elif [[ -x "${CHKCONFIG}" ]]; then
    echo -e "\nSystem is an RedHat-based, using chkconfig for boot files installation"
    ${CHKCONFIG} --add collector
    check_return_code \
      $? \
      "WARNING: chkconfig failed, you should manually configure collector to run at boot!"

  else
    echo -e "\nWARNING: Unable to configure collector for running at boot!"
    echo "You should configure it manually."
  fi
fi

# Removing support files
rm -f /tmp/collector.conf
 
# Finished
echo
echo "*** Installation successfull! ***"
echo 
echo If you do not have a database please cd into:
echo ${SHARE_DIR}/database
echo and execute ./install.sh
echo
echo If you are updating the collector and you
echo already have a database please cd into:
echo ${SHARE_DIR}/database
echo and execute ./update.sh

if [[ "${use_prev_conf}" != "y" ]]; then
  echo
  echo Please, edit the configuration file ${ETC_DIR}/collector.conf
  echo and make changes according to your needs.
  echo
  echo After this, you can start collector
fi

echo -e "\n*** Bye ***\n"

### MAIN PROGRAM END ###
